#!/bin/sh


##        Licensed Materials - Property of IBM
##        (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved
##
##       US Government Users Restricted Rights - Use, duplication,
##       or disclosure restricted by GSA ADP Schedule Contract with
##       IBM Corp.
##

##
##   TFST 1.3 - script for upgrading the endpoint proxy from version 1.2 to
##   the current version or for applying a patch to an installed version of
##   1.3 or 1.3.1.
##

version_major="1"
version_minor="3"
version_point="1"
version="$version_major.$version_minor.$version_point"
instance="epproxy"
repository_dir="/etc/Tivoli/TFST/$version/installed"
patchPrefix="1.3.1-TFS-"
patchNumber="0001"
patchID="$patchPrefix$patchNumber"
defaultPrevPatch="0001"

if echo -n | egrep -e -n >/dev/null ; then
    dashn=""
else
    dashn=-n
fi



print_os_name()
{
	if [ -f /bin/sun ] && /bin/sun ; then
		echo solaris
	elif [ `uname` = "AIX" ] ; then
		echo aix
	elif [ `uname` = "HP-UX" ] ; then
		echo hpux
	else
		if [ `uname` = "Linux" ] ; then
			echo linux
		else
			uname
		fi
	fi
}


check_response()
{
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ||
         [ "$1" = "Y" -a X${ans} = X ] ; then
		return 0
	fi

	return 1
}

check_path()
{
	if [ ! -d $1 ] ; then
		return 1
	fi

	if [ ! -f "$1/epproxy" ] ; then
		return 1
	fi

	if [ ! -f "$1/epproxy.cfg" ] ; then
		return 1
	fi

	return 0
}

check_installed()
{
    RepDir13="/etc/Tivoli/TFST/1.3/installed"
    RepDir131="/etc/Tivoli/TFST/$version/installed"
	
    instDirFound=0

    # check if the installation repository directory exists
    if [ -d $RepDir131 ] ; then
        if [ -f "$RepDir131/$instance" ] ; then
            installedVersion=$version
            instDirFound=1
        fi
    fi

    if [ $instDirFound = 0 ] ; then
        if [ -d $RepDir13 ] ; then
            if [ -f "$RepDir13/$instance" ] ; then
                installedVersion="1.3"
                instDirFound=1
            fi
        fi
    fi

    return $instDirFound
}

auto_start_remove()
{
OSNAME=`print_os_name`

case "$OSNAME" in
	aix)
		RCFILE=/etc/rc.$instance
		RCTAB=$instance

        set e+

		if [ -f $RCFILE ] ; then
			rm -rf $RCFILE
        fi

        rmitab $RCTAB

		set -e
	    ;;

	*)
	    for dir in /etc/rc.d/init.d /etc/init.d /usr/sbin/init.d /sbin/init.d; do
	        [ -d $dir ] && MRCD=$dir
	    done

	    rm -f $MRCD/$instance*
	    if [ -d /var/lock/subsys ]; then
		rm -f /var/lock/subsys/$instance
	    fi

	    find /etc/rc* -name S*$instance -exec rm {} \;
	    find /etc/rc* -name K*$instance -exec rm {} \;
	
    esac

    return 0
}

get_YN_response()
{
    ok=0
    response=0
    while [ $ok = 0 ] ; do
        echo $dashn $1
        read ans

        if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
           [ X${ans} = Xyes ] || [ X${ans} = XYES ] ; then
            response=0
            ok=1
        fi

        if [ X${ans} = Xn ] || [ X${ans} = XN ] ||
           [ X${ans} = Xno ] || [ X${ans} = XNO ] ; then
            response=1
            ok=1
        fi
    done

    return $response
}

get_install_dir()
{
    instDir=`grep installation_dir $installFile`

    if [ X$instDir = X ] ; then
        echo "The Endpoint Proxy installation directory cannot be determined."
        echo "Cannot continue, exiting..."
        exit 1
    else
        instDir=`grep installation_dir $installFile | cut -d= -f2`
    fi

    # IY47957: trailing slashes cause problems so remove them
    #    (but not for bare slash ("/"))
    instDir=`echo $instDir | sed 's,\(.\)//*$,\1,'`

    check_path ${instDir}
    if [ $? != 0 ] ; then
        echo "The Endpoint Proxy was not found in the directory $instDir."
        echo "Nothing to patch, exiting..."
        exit 1
    else
        install_dir=${instDir}
    fi
}

get_username()
{
    # TFST 1.3 GA does not have 'runas' line in the install history file;
    # the userID that endpoint proxy will run as is the owner of the
    # endpoint proxy binary.
    # Patched versions of TFST 1.3 should have a 'runas' line in
    # the install history file to identify the userID the endpoint proxy
    # runs as, since install versions after TFST 1.3 GA will set the owner
    # of the endpoint proxy binary to root.
    runasName=`grep runas $installFile`
    if [ X${runasName} = X ] ; then
        username=`ls -l $install_dir/$instance | awk '{ print $3 }'`
    else
        username=`grep runas $installFile | cut -d= -f2`
    fi

    if [ X${username} = X ] ; then
        echo "Unable to identify the user of the Endpoint Proxy directory ($install_dir/$instance)."
        echo "Cannot continue, exiting..."
        exit 1
    fi
}

check_patch_level()
{
    installFile="/etc/Tivoli/TFST/$1/installed/$instance"
    _patchPrefix="$1-TFS-"

    # The $installFile for TFST 1.3 GA does not contain the $patchPrefix
    # string.  Installs or upgrades of subsequent patches will write
    # the patchID to the $installFile.
    patchApplied=`grep $_patchPrefix $installFile`

#    if [ X'$patchApplied' = X ] ; then
#       previousPatch=$_patchPrefix$defaultPrevPatch
     if [ $? != 0 ] ; then
        # grep failed to find $_patchPrefix in $installFile
        if [ $1 != $version ] ; then
            # assume this is 1.3 GA
            previousPatch=$_patchPrefix$defaultPrevPatch
        else
            # this is a 1.3.1 install, but the patch level is unknown.
            # Ask if install should continue.
            echo "The patch level of the existing Endpoint Proxy cannot be determined."
            get_YN_response "Do you want to continue installing this patch? [Y/N] ";
            if [ $? = 0 ] ; then
                previousPatch=$_patchPrefix"Unknown"
            else
                echo "The patch will not be applied."
                exit 1
            fi
        fi
    else
        # we're in this leg if _patchPrefix (e.g. 1.3.1-TFS) was found
        # $installFile.  Get the patch string from $installFile and
        # then parse it further.  The following code breaks the
        # patch string into its component parts.

        # $installedPatch will be something like 1.3-TFS-0002
        installedPatch=`grep version $installFile | awk '{ print $2 }'`

        # $installedPatchNum will be something like 0002
        installedPatchNum=`echo $installedPatch | cut -d- -f3`

        # $instPfx1 will be something like 1.3.
        instPfx1=`echo $installedPatch | cut -d- -f1`
        installedPatchPrefix="$instPfx1-TFS-"

        # $installedPatchNum should never be blank, but handle it just in case.
        if [ X$installedPatchNum = X ] ; then
            echo "The installed patch level is $_patchPrefix0001."
            previousPatch="$_patchPrefix0001"

        else
            previousPatch=$_patchPrefix$installedPatchNum
            echo "The installed patch level is $previousPatch."

            # here, installedPatchNum could be 0002 (for 1.3-TFS-0002).
            # this will be less than $patchNumber (0001).
            # Don't do the next two comparisons if $1 != $version
            if [ $1 = $version ] ; then

                if [ $installedPatchNum = $patchNumber ] ; then
                    echo "The patch level of the installed Endpoint Proxy ($_patchPrefix$installedPatchNum) is the same as this patch."
                    get_YN_response "Do you want to continue installing this patch? [Y/N] ";
                    if [ $? = 0 ] ; then
                        previousPatch=$_patchPrefix$installedPatchNum
                    else
                        echo "The patch will not be applied."
                        exit 1
                    fi
                elif [ $installedPatchNum -gt $patchNumber ] ; then
                    echo "The patch level of the installed Endpoint Proxy ($_patchPrefix$installedPatchNum) is higher than this patch."
                    echo "Installing this patch will result in patch $patchID replacing what is currently installed ($_patchPrefix$installedPatchNum)."
                    get_YN_response "Do you want to continue installing this patch? [Y/N] ";
                    if [ $? = 0 ] ; then
                        previousPatch=$_patchPrefix$installedPatchNum
                    else
                        echo "The patch will not be applied."
                        exit 1
                    fi
                fi
            fi
        fi
    fi

    # set install_dir (endpoint proxy instance install location)
    get_install_dir $installFile

    # set username (userID that endpoint proxy instance will run as)
    get_username $installFile

    installedRepositoryFile=$installFile
}

################################################
#
#    install script starts here
#
################################################

clear
work_dir=`pwd`

#
# check platform
#
os_name=`print_os_name`
if [ X${os_name} != Xhpux ] && [ X${os_name} != Xaix ] && [ X${os_name} != Xsolaris ] && [ X${os_name} != Xlinux ] ; then
	echo "Platform $os_name is not supported."
 	exit 1
fi

#
# ask for license agreement
#
ok=0

echo "To install or upgrade the Tivoli Endpoint Proxy, you must accept the agreement written in the License file."

get_YN_response "If you accept the agreement, enter Y.  [Y/N]: ";
if [ $? = 1 ] ; then
    exit 1
fi

#
# See if 1.3 is installed.  If so, patch.  Otherwise upgrade from 1.2
# to this patch level of 1.3.
#
unknown="unknown"
install_dir=$unknown
username=$unknown
previousPatch=$unknown
installedRepositoryFile=$unknown
installedVersion=$unknown
install13Patch=0
check_installed;
### if check_installed() returns 1, $installedVersion will indicate
### the installed version number (currently checking only for 1.3 & 1.3.1)
if [ $? = 1 ] ; then
     echo "An earlier version of Tivoli Endpoint Proxy, version 1.3 or later, is already installed."
     echo "Checking Endpoint Proxy patch level..."
       install13Patch=1
fi

if [ $install13Patch = 1 ] ; then
    # On return from check_patch_level, $install_dir, $username,
    # $previousPatch, $installedRepositoryFile, and $instance have been set.
    # check_patch_level exits if the currently installed patch level
    # is >= this patch level and the user declines to proceed.
    check_patch_level $installedVersion;
fi

#
# If upgrade from 1.2, ask for installation directory
#
ok=0
if [ $install13Patch = 0 ] ; then
    while [ $ok = 0 ] ; do
	echo $dashn "Endpoint Proxy 1.2 installation directory: "
	read ans
	if [ X${ans} != X ] ; then
		# check if the path exists
		try_path=${ans}
		check_path ${try_path}
		if [ $? != 0 ] ; then
			echo $dashn "The endpoint proxy was not found in the directory: $try_path"
		else
			install_dir=${try_path}

			# IY47957: trailing slashes cause problems so remove them
                        #    (but not for bare slash ("/"))
                        install_dir=`echo $install_dir | sed 's,\(.\)//*$,\1,'`

			ok=1
		fi
	fi
    done

    #
    # get the user name
    #
    username=`ls -l $install_dir/epproxy | awk '{ print $3 }'`
    if [ X${username} = X ] ; then
	echo $dashn "Unable to identify the user of the directory you specified."
	exit 1
    fi
fi

#
# Confirm upgrade/patch
#
ok=0
while [ $ok = 0 ] ; do
   if [ $install13Patch = 1 ] ; then
       backupDir=${install_dir}.${previousPatch}
       echo
       echo "The configuration and all the data of the current Endpoint Proxy installation will be backed up to '$backupDir'."
       echo "If the directory '$backupDir' exists, the directory and all of its current contents will be removed."
       echo "Configuration data from the current Endpoint Proxy installation will be used by the patch."
       get_YN_response "Do you want to apply the patch now? [Y/N] ";
   else
       backupDir=${install_dir}.1.2
	echo
	echo "A previous version of the Endpoint Proxy has been detected on the machine."
	echo "The configuration and all the data of the previous version will be backed up to '${install_dir}.1.2'."
	echo "It will be used by the new version."
       get_YN_response "Do you want to upgrade to the new version? [Y/N] ";
   fi

   if [ $? = 1 ] ; then
       exit 1
   fi
done

#
# back up the appropriate directory tree
#
if [ -d "${backupDir}" ] ; then
	rm -rf "${backupDir}"
fi
cp -rf ${install_dir} "${backupDir}"

#
# remove old directory (except cfg)
#
$install_dir/epproxy.sh stop > /dev/null
rm -f $install_dir/epproxy
rm -f $install_dir/wproxy
if [ X${os_name} != Xaix ] ; then
	rm -f $install_dir/*.so
else
	rm -f $install_dir/*.a
fi
rm -f $install_dir/*.log
rm -f $install_dir/*.pid
rm -f $install_dir/epproxy.sh
rm -f $install_dir/setup_env.sh
rm -f $install_dir/C/*
rmdir $install_dir/C
if [ $install13Patch = 1 ] ; then
    rm -f $install_dir/*.log.bak > /dev/nul
    rm -f $install_dir/uninstall.sh > /dev/nul
    rm -rf $install_dir/de > /dev/nul
    rm -rf $install_dir/es > /dev/nul
    rm -rf $install_dir/fr_FR > /dev/nul
    rm -rf $install_dir/it > /dev/nul
    rm -rf $install_dir/ja_JP > /dev/nul
    rm -rf $install_dir/ko > /dev/nul
    rm -rf $install_dir/pt_BR > /dev/nul
    rm -rf $install_dir/zh_CN > /dev/nul
    rm -rf $install_dir/zh_TW > /dev/nul
fi
auto_start_remove

#
# launch the silent install
#
# if upgrading from 1.3 to 1.3.1, need to remove the existing
# install file /etc/Tivoli/TFST/1.3/installed/epproxy.
if [ $install13Patch = 1 -a $version != $installedVersion ] ; then
    # pass in name of existing repository file so install can remove it
    ./install.sh -s ${install_dir} ${username} ${patchID} ${installedRepositoryFile}
else
    ./install.sh -s ${install_dir} ${username} ${patchID}
fi

if [ $install13Patch = 1 ] ; then
    echo "Endpoint Proxy patch $patchID applied successfully."
else
    echo "Endpoint Proxy successfully upgraded."
fi

#
# ask for endpoint proxy start-up
#
ok=0
default="Y"
while [ $ok = 0 ] ; do
	echo $dashn "Start the endpoint proxy? [Y/N] [default=$default]: "
	check_response $default
	if [ $? = 0 ] ; then
		nohup ${install_dir}/epproxy.sh start 1>/dev/null 2>&1 &
	else
		echo "Endpoint proxy not started"
	fi
	ok=1
done
