#!/bin/sh

##        Licensed Materials - Property of IBM
##        (C) Copyright IBM Corp. 2000, 2003 All Rights Reserved
##
##       US Government Users Restricted Rights - Use, duplication,
##       or disclosure restricted by GSA ADP Schedule Contract with
##       IBM Corp.
##
##   TFST 1.3.1 - event sink uninstall script
##

version="1.3.1"
instance="eventsink"
repository_dir="/etc/Tivoli/TFST/$version/installed"
repository_root="/etc/Tivoli/TFST"

if echo -n | egrep -e -n >/dev/null ; then
	dashn=""
else
	dashn=-n
fi

print_interp()
{
	if [ -f /bin/sun ] && /bin/sun ; then
	        set `uname -a`
	        ch=`echo $3 | cut -c1`
	        case $ch in
	        4)      echo sunos4 ;;
	        5)      echo solaris2 ;;
	        *)      echo UNKNOWN ;;
	        esac
	elif [ `uname` = "AIX" ] ; then
	        case `uname -vr` in
	        "2 3") echo aix3-r2 ;;
	        "1 4") echo aix4-r1 ;;
	        "2 4") echo aix4-r1 ;;
	        "3 4") echo aix4-r1 ;;
	        *) exit 1 ;;
	        esac
	elif [ `uname -s` = "HP-UX" ]; then
			case `uname -r` in
			[A-Z].09.0[0-9]) echo hpux9 ;;
			[A-Z].10.0[0-9]) echo hpux10 ;;
			[A-Z].10.2[0-9]) echo hpux10 ;;
			[A-Z].11.[0-9][0-9]) echo hpux10 ;;
			*) exit 1 ;;
			esac
	else
	      if [ `uname` = "Linux" ] ; then
			echo linux-ix86
		else
			uname
		fi
		
	fi
}

print_os_name()
{
	if [ -f /bin/sun ] && /bin/sun ; then
		echo solaris
	elif [ `uname` = "AIX" ] ; then
		echo aix
	elif [ `uname` = "HP-UX" ] ; then
		echo hpux
	else
		if [ `uname` = "Linux" ] ; then
			echo linux
		else
			uname
		fi
	fi
}

check_response()
{
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ||
         [ "$1" = "Y" -a X${ans} = X ] ; then
		return 0
	fi

	return 1
}

auto_start_remove()
{
instance=eventsink

INTERP=${interp}

case "$INTERP" in
	aix3-r2|aix4-r1)
		RCFILE=/etc/rc.$instance
		RCTAB=$instance

        set e+

		if [ -f $RCFILE ] ; then
			rm -rf $RCFILE
        fi

        rmitab $RCTAB
        rm -f /etc/inittab.before.$instance
        set -e
	    ;;

	hpux*)
	    for dir in /etc/rc.d/init.d /etc/init.d /usr/sbin/init.d /sbin/init.d; do
	        [ -d $dir ] && MRCD=$dir
	    done

	    rm -f $MRCD/$instance*
	    if [ -d /var/lock/subsys ]; then
		rm -f /var/lock/subsys/$instance
	    fi

	    find /sbin/rc* -name S*$instance -exec rm {} \;
	    find /sbin/rc* -name K*$instance -exec rm {} \;
		;;
	*)
	    for dir in /etc/rc.d/init.d /etc/init.d /usr/sbin/init.d /sbin/init.d; do
	        [ -d $dir ] && MRCD=$dir
	    done

	    rm -f $MRCD/$instance*
	    if [ -d /var/lock/subsys ]; then
		rm -f /var/lock/subsys/$instance
	    fi

	    find /etc/rc* -name S*$instance -exec rm {} \;
	    find /etc/rc* -name K*$instance -exec rm {} \;
	
    esac

    return 0
}

#
# main starts here
#

clear

#
# the setting of this variable is done by
# the install script when started
#
work_dir=!!INSTALL_DIR!!

interp=`print_interp`

# check the os
os_name=`print_os_name`
if [ X${os_name} != Xhpux ] && [ X${os_name} != Xaix ] && [ X${os_name} != Xsolaris ] && [ X${os_name} != Xlinux ] ; then
	echo "Platform $os_name is not supported."
	exit 1
fi

# ask the confirmation
ok=0
while [ $ok = 0 ] ; do
	echo	
	echo $dashn "Are you sure you want to remove the Tivoli Event Sink [Y/N]: "
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ; then
		ok=1
	fi

	if [ X${ans} = Xn ] || [ X${ans} = XN ] ||
         [ X${ans} = Xno ] || [ X${ans} = XNO ] ; then
		exit 1
	fi
done

# check that the install dir has been properly set
if [ ! -d $work_dir ]; then
	echo "The installation directory is unknown."
	exit 1
fi

cd $work_dir

# stop the application
echo $dashn "Stopping the event sink... "
./eventsink.sh stop > /dev/null
echo "OK"

# remove the application
echo $dashn "Removing the event sink... "
rm -f eventsink*   > /dev/null
rm -f uninstall.sh > /dev/null
rm -rf C           > /dev/null
rm -rf it          > /dev/null
rm -rf de          > /dev/null
rm -rf es          > /dev/null
rm -rf fr_FR       > /dev/null
rm -rf pt_BR       > /dev/null
rm -rf zh_CN       > /dev/null
rm -rf zh_TW       > /dev/null
rm -rf ja_JP       > /dev/null
rm -rf ko          > /dev/null
cd ..
# remove $work_dir if it is empty
if [ `ls $work_dir | wc -l` = 0 ] ; then
    rm -rf $work_dir > /dev/null
fi
echo "OK"

# remove the auto start stuff
echo $dashn "Removing the automatic restart... "
auto_start_remove
echo "OK"

# remove the registration from the repository
echo $dashn "Removing the application registration... "
rm -f $repository_dir/$instance > /dev/null
# remove the registration directory if empty
if [ `ls $repository_dir | wc -l` = 0 ] ; then
   rm -rf $repository_dir > /dev/null
   # TFST 1.3.1 install creates the $repository_root directory.
   # If it has no subdirectories other than the one for this version of TFST, remove it.
   if [ `ls $repository_root | wc -l` = 1 ] && [ "`ls $repository_root`" = "$version" ] ; then
      rm -rf $repository_root > /dev/null
   fi
fi
echo "OK"

echo "Uninstall completed"
echo
