#!/bin/sh

##
##        Licensed Materials - Property of IBM
##        (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved
##
##       US Government Users Restricted Rights - Use, duplication,
##       or disclosure restricted by GSA ADP Schedule Contract with
##       IBM Corp.
##

##
##   TFST 1.3.1 - event sink install script
##

version="1.3.1"
instance="eventsink"
patchID="1.3.1-TFS-0001"
repository_dir="/etc/Tivoli/TFST/$version/installed"

if echo -n | egrep -e -n >/dev/null ; then
	dashn=""
else
	dashn=-n
fi

# create the installation repository directory
if [ ! -d $repository_dir ] ; then
	mkdir -p $repository_dir
fi

if [ ! -d $repository_dir ] ; then
	echo "Unable to create the repository folder '$repository_dir'"
	return 0
fi

print_interp()
{
# Note: framework installs the mrt stuff under aix4-r1 also for AIX 5

       if [ -f /bin/sun ] && /bin/sun ; then
               set `uname -a`
               ch=`echo $3 | cut -c1`
               case $ch in
               4)      echo sunos4 ;;
               5)      echo solaris2 ;;
               *)      echo UNKNOWN ;;
               esac
       elif [ `uname` = "AIX" ] ; then
               case `uname -vr` in
               "2 3") echo aix3-r2 ;;
               "1 4") echo aix4-r1 ;;
               "2 4") echo aix4-r1 ;;
               "3 4") echo aix4-r1 ;;
               "1 5") echo aix4-r1 ;;
               *) echo aix4-r1 ;;
               esac
       elif [ `uname -s` = "HP-UX" ]; then
				case `uname -r` in
				[A-Z].09.0[0-9]) echo hpux9 ;;
				[A-Z].10.0[0-9]) echo hpux10 ;;
				[A-Z].10.2[0-9]) echo hpux10 ;;
				[A-Z].11.[0-9][0-9]) echo hpux10 ;;
				*) exit 1 ;;
				esac
       else
              if [ `uname` = "Linux" ] ; then
                     echo linux-ix86
              else
                     uname
              fi
       fi
}

print_os_name()
{
	if [ -f /bin/sun ] && /bin/sun ; then
		echo solaris
	elif [ `uname` = "AIX" ] ; then
		echo aix
	elif [ `uname` = "HP-UX" ] ; then
		echo hpux
	else
		if [ `uname` = "Linux" ] ; then
			echo linux
		else
			uname
		fi
	fi
}


check_response()
{
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ||
         [ "$1" = "Y" -a X${ans} = X ] ; then
		return 0
	fi

	return 1
}

check_path()
{
	if [ -d $1 ] ; then
		return 0
	fi

	return 1
}

make_path()
{
	check_path $1
	if [ $? != 0 ] ; then
		mkdir -p $1
	fi
	check_path $1
	return $?
}

#
# This function replaces the !!xxx!! patterns
# in the uninstall script.
#
# $1 is the full path uninstall file
#
patch_uninstall_script()
{
	if [ ! -f $1 ] ; then
		echo "Uninstall file '$1' not found, the uninstall script will be not available"
		return 0
	fi

  	sed_cmd=" sed -e s#!!INSTALL_DIR!!#$install_dir#g "
	$sed_cmd $1 > $1.new
	rm -f $1
	mv $1.new $1

	return 1		
}


write_instance()
{
	installed_file=$repository_dir/$1

	if [ -f $installed_file ] ; then
		rm -f $installed_file
	fi

	# create the installation info file
cat >$installed_file <<EOF
installation_dir=$install_dir
installation_time=`date`
version=$version $patchID
runas=$username
EOF

	return 1		
}

auto_start()
{
proxy=eventsink
OSNAME=`print_os_name`

ETRC=$install_dir/$proxy.sh
CRCF="/etc/rc3.d/S99$proxy \
      /etc/rc0.d/K49$proxy \
      /etc/rc1.d/K49$proxy \
      /etc/rc2.d/K49$proxy"
DEVNULL=/dev/null
DATE=`date`
# add /usr/sbin to path for AIX to find mkitab
PATH=$PATH:/usr/sbin


case "$OSNAME" in

linux)
    # SuSE looks for kill scripts in prev level. All other look in new runlevel
    if [ -f /etc/SuSE-release ]; then
      CRCF="/etc/rc.d/rc2.d/S99$proxy \
            /etc/rc.d/rc3.d/S99$proxy \
            /etc/rc.d/rc4.d/S99$proxy \
            /etc/rc.d/rc5.d/S99$proxy \
            /etc/rc.d/rc2.d/K9$proxy \
            /etc/rc.d/rc3.d/K9$proxy \
            /etc/rc.d/rc4.d/K9$proxy \
            /etc/rc.d/rc5.d/K9$proxy"
    else
      CRCF="/etc/rc.d/rc2.d/S99$proxy \
            /etc/rc.d/rc3.d/S99$proxy \
            /etc/rc.d/rc4.d/S99$proxy \
            /etc/rc.d/rc5.d/S99$proxy \
            /etc/rc.d/rc0.d/K9$proxy \
            /etc/rc.d/rc1.d/K9$proxy \
            /etc/rc.d/rc6.d/K9$proxy"
    fi
    ;;
hpux)
      CRCF="/sbin/rc3.d/S999$proxy \
            /sbin/rc0.d/K9$proxy \
            /sbin/rc1.d/K9$proxy \
            /sbin/rc2.d/K9$proxy"
    ;;
esac

#echo "RC files are: $CRCF"

#################
# Setup autostart
#################
case "$OSNAME" in
	aix)
		RCFILE=/etc/rc.$proxy
		RCTAB=$proxy
		INITTAB=/etc/inittab
		INITSAVE=$INITTAB.before.$proxy

		if [ ! -f $RCFILE ]
		then
			set +e
			cat >> $RCFILE <<EOF
#!/bin/sh
#
#		Start the Tivoli $proxy
#
if [ -f $ETRC ]; then
    $ETRC start
fi
EOF
			# If the creation of the /etc/rc.$proxy file fails, then
			# we need to exit, cleaning up any changes we might have
			# made.
			if [ $? -ne 0 ]
			then
				rm -rf $RCFILE
				echo "Unable to install Tivoli $proxy"
				return 1
			fi

			chmod 0774 $RCFILE
			if [ $? -ne 0 ]
			then
				rm -rf $RCFILE
				echo "Unable to install Tivoli $proxy"
				return 1
			fi

			if lsitab rcnfs >/dev/null 2>&1
			then
				RCENTRY=rcnfs
			else
				RCENTRY=rctcpip
			fi

			#
			#  Okay - we've successfully added the rc startup file,
			# need to modify /etc/inittab
			#
			cp $INITTAB $INITSAVE
			mkitab "$RCTAB:2:wait:$RCFILE > /dev/console 2>&1 # Tivoli $proxy"
			if [ $? -ne 0 ]
				then
				rmitab $RCTAB
				rm -rf $RCFILE
				cp $INITSAVE $INITTAB
				rm -rf $INITSAVE
				echo "Unable to modify /etc/inittab, removing $proxy installation"
				return 1
			fi
			set -e
		fi
	    ;;
	*)
	    for dir in /etc/rc.d/init.d /etc/init.d /usr/sbin/init.d /sbin/init.d; do
	        [ -d $dir ] && MRCD=$dir
	    done

	    if [ "$OSNAME" = "linux" ]; then
	        MRCF="$MRCD/$proxy"
		if [ -d /var/lock/subsys ]; then
			touch /var/lock/subsys/$proxy
		fi
	    else
	        MRCF="$MRCD/$proxy.rc"
	    fi

	    if [ ! -d $MRCD ]; then
	        MRCF="$ETRC"
	    fi
	    if test -f $MRCF && cmp -s $MRCF $ETRC >$DEVNULL 2>&1; then
	        echo "$MRCF Tivoli $proxy daemon autostart file is already present."
	    else
	        rm -f $MRCF
	    	cp -p $ETRC $MRCF
	    fi
	    for i in $CRCF; do
		if [ -f $i ]; then
		    rm -f $i;
		fi
		# the convention for hpux and sgi is to always
		# symlink startup scripts from init.d/script to
		# the corresponding rc?.d/script
		case "$OSNAME" in
		    hpux|linux)
			di=`dirname $i`
			if [ X"$di" != X"" -a -d "$di" ] ; then
				ln -s $MRCF $i
			fi
			;;
		    *)
	        	ln $MRCF $i >$DEVNULL 2>&1
			if [ $? -ne 0 ]; then
			    ln -s $MRCF $i
			fi
			;;
		esac
	    done
            ;;
    esac

    return 0
}

install_files()
{
        # set permissions on installed files
        chown $username $install_dir
        chmod 750 $install_dir
        cp eventsink uninstall.sh $install_dir/
        chmod 555 $install_dir/eventsink

        for langDir in C it de es fr_FR pt_BR zh_CN zh_TW ja_JP ko ; do
            cp -rf $langDir $install_dir/
            chmod 755 $install_dir/$langDir
            chmod 644 $install_dir/$langDir/*
        done

        patch_uninstall_script $install_dir/uninstall.sh
        chmod 500 $install_dir/uninstall.sh
}

write_cfg_file()
{
cat >$install_dir/eventsink.cfg <<EOF
# The [SENDING] section lists options for sending events to the
# Tivoli Enterprise Console server.
[SENDING]
# The full path to the dat directory of the Tivoli endpoint
lcf-datdir=$lcf_datdir

# Maximum buffer size, in bytes, of the package that the event sink
# sends to the Tivoli Enterprise Console server. (default = 40000)
max-size-buffer=$max_buf_sz

# The maximum number of events that the event sink sends to the
# Tivoli Enterprise Console server in a single package. (default = 50)
max-num-events-to-send=$max_num_evts

# Minimum interval, in seconds, between sending packages of events to
# the Tivoli Enterprise Console gateway. To send packages immediately,
# specify 0 (default = 1)
delay-time=1

# Timeout, in seconds, by which the event sink sends events if neither
# the maximum buffer size nor the maximum number of events is
# reached (default = 30)
caching-timeout=30


# The [RECEPTION] section lists options for receiving events from the
# non-TME adapters.
[RECEPTION]

# Port number on the endpoint where the event sink receives
# events (default = 9444)
port=$listening_port

# Maximum number of threads that the event sink can have with the
# non-TME adapters. (default = 100)
max-sessions=100

# Timeout interval (in seconds) for TCP/IP operations to succeed/fail
tcpip-timeout=120

# Memory size, in kilobytes, in which events get cached in the event sink.
# The event sink stops receiving events when this value is
# reached until it sends the events to the Tivoli Enterprise Console
# gateway (default = 1024)
max-ram-cache=1024


# The [EIF] section lists options for the Tivoli Enterprise
# Integration Facility. Any configuration value that EIF supports
# can be specified here.
[EIF]

# Specifies the name of the host on which the event server is installed.
# (default = @EventServer)
#ServerLocation=

# Specifies the port number on which the event server listens for events.
#ServerPort=

# The maximum size, in kilobytes, that EIF uses to group events for
# bulk upload (default = 64)
BufEvtMaxSize=64

# Full path to file in which the EIF saves events that it
# temporarily cannot send (default = <install_dir>/eventsink.cache)
BufEvtPath=$install_dir/eventsink.cache

# Maximum buffer size, in bytes, of the package that the EIF
# sends to the Tivoli Enterprise Console server. (default = 40000)
MaxPacketSize=$max_buf_sz

# The [LOG] section lists Event Sink logging options.
[LOG]

# Full path of the log file where event sink messages are
# written (default = <install_dir>/eventsink.log)
log-file=$install_dir/eventsink.log

# The level of detail in the log (default = 3)
debug-level=3

# The max size of the log file in MegaBytes (if 0 the log file size is unlimited) (default = 1)
max-size=1
EOF

chmod 644 $install_dir/eventsink.cfg
}

write_start_script()
{
nfile=unlimited
ulimit_solaris2="ulimit -n $nfile"
ulimit_linux="ulimit -n 32768"
# for HP ulimit -n is not available
ulimit_hpux=""

if [ X${os_name} = Xsolaris ] ; then
    ulimit_cmd=${ulimit_solaris2}
fi

if [ X${os_name} = Xlinux ] ; then
	ulimit_cmd=${ulimit_linux}
fi

if [ X${os_name} = Xhpux ] ; then
    ulimit_cmd=${ulimit_hpux}
	nohup_cmd=nohup
fi

logfile_path=$install_dir/eventsink.log

# create the script to run the proxy
cat >$install_dir/eventsink.sh <<EOF
#!/bin/sh
INTERP=`print_interp`
export INTERP
INSTALL_DIR_PATH=$install_dir
export INSTALL_DIR_PATH
LCFROOT="$lcf_datdir/../.."
export LCFROOT
LCF_DATDIR="$lcf_datdir"
export LCF_DATDIR
LCF_BINDIR="\$LCFROOT/bin/\$INTERP/mrt"
export LCF_BINDIR
LCF_LIBDIR="\$LCFROOT/lib/\$INTERP"
export LCF_LIBDIR
LCF_CATDIR="\$LCFROOT/generic/msg_cat"
export LCF_CATDIR
LCF_TOOLSDIR="\$LCFROOT/bin/\$INTERP/tools"
export LCF_TOOLSDIR
PATH=/bin:/usr/bin:/usr/ucb:/usr/sbin:\$LCF_BINDIR:\$LCF_DATDIR:\$PATH
export PATH
NLSPATH=\$INSTALL_DIR_PATH/%L/%N.cat:\$INSTALL_DIR_PATH/%l/%N.cat:\$INSTALL_DIR_PATH/C/%N.cat:\$LCF_CATDIR/%L/%N.cat:\$LCF_CATDIR/%l/%N.cat:\$LCF_CATDIR/C/%N.cat\${NLSPATH:+":\$NLSPATH"}
export NLSPATH
TISDIR="\$LCF_DATDIR"
export TISDIR

# AIX
LIBPATH=\$LCF_LIBDIR:/usr/lib\${LIBPATH:+":\$LIBPATH"}
export LIBPATH
# Solaris, Linux
LD_LIBRARY_PATH=\$LCF_LIBDIR:/usr/openwin/lib:/usr/lib:/usr/ucblib\${LD_LIBRARY_PATH:+":\$LD_LIBRARY_PATH"}
export LD_LIBRARY_PATH
# HP-UX
SHLIB_PATH=\$LCF_LIBDIR:/usr/lib\${SHLIB_PATH:+":\$SHLIB_PATH"}
export SHLIB_PATH
${ulimit_cmd}

[ -d "\$INSTALL_DIR_PATH" ] && cd \$INSTALL_DIR_PATH

case "\$1" in
	
	stop_msg)
		shift
		echo "Stopping the eventsink daemon"
		;;
	stop)
		shift
		if [ -f \$INSTALL_DIR_PATH/eventsink.pid ] ; then
			pid=\`sed -n 1p \$INSTALL_DIR_PATH/eventsink.pid\`
			if [ -n "\$pid" ] ; then
				
				pidlist=\`ps -el | grep eventsin | grep -v grep | awk '{print \$4}'\`
				
				for temppid in \$pidlist
				do
					if [ \$temppid -eq \$pid ] ; then
						rm -f \$INSTALL_DIR_PATH/eventsink.pid
						kill -15 \$pid
						exit \$?
					fi
				done
				echo "The process id in \$INSTALL_DIR_PATH/eventsink.pid is not an eventsink process id."
				echo "Unable to get process id of the eventsink daemon (pid)."
			else
				echo "Unable to get process id of the eventsink daemon (pid)."
				exit 1
			fi
		else
			echo "Unable to get process id of the eventsink daemon (eventsink.pid)."
			exit 1
		fi
		;;
	start_msg)
		shift
		echo "Starting the eventsink daemon"
		;;
	start|*)
		if [ "\$1" = "start" ] ; then
			shift
		fi

		if [ -f \$INSTALL_DIR_PATH/eventsink.pid ] ; then
			pid=\`sed -n 1p \$INSTALL_DIR_PATH/eventsink.pid\`
			if [ -n "\$pid" ] ; then
				
				pidlist=\`ps -el | grep eventsin | grep -v grep | awk '{print \$4}'\`
				
				for temppid in \$pidlist
				do
					if [ \$temppid -eq \$pid ] ; then
						echo "eventsink for this environment already running under pid \$pid"
						exit 1
					fi
				done
			fi
		fi

		currentUser=\`whoami\`
		if [ "\$currentUser" = "$username" ] ; then
			cd \${INSTALL_DIR_PATH};
			$nohup_cmd \$INSTALL_DIR_PATH/eventsink \$INSTALL_DIR_PATH 2>&1 | grep -v "syntax error in configuration file" &
		else
			$nohup_cmd su - $username -c "cd \${INSTALL_DIR_PATH}; INTERP=\$INTERP LCFROOT=\$LCFROOT LCF_DATDIR=\$LCF_DATDIR LCF_BINDIR=\$LCF_BINDIR LCF_LIBDIR=\$LCF_LIBDIR LCF_CATDIR=\$LCF_CATDIR LCF_TOOLSDIR=\$LCF_TOOLSDIR PATH=\$PATH NLSPATH=\$NLSPATH TISDIR=\$TISDIR LIBPATH=\$LIBPATH LD_LIBRARY_PATH=\$LD_LIBRARY_PATH SHLIB_PATH=\$SHLIB_PATH \$INSTALL_DIR_PATH/eventsink \$INSTALL_DIR_PATH 2>&1 | grep -v 'syntax error in configuration file'" &
		fi


		;;
esac
EOF

# set permissions on the script
chmod 500 $install_dir/eventsink.sh
}


################################################
#
#    install script starts here
#
################################################

work_dir=`pwd`
os_name=`print_os_name`

#
# Check if the installation must be silent (-s).
# In this case is expected as
#
#	2nd parameter: the installation directory
#	3nd parameter: the user name
#      4th parameter: lcf_datdir
#      5th parameter: the patch ID (optional)
#      6th parameter: existing repository file to be deleted
#                     if install succeeds
#
# and no configuration file will be written.
#
if [ X$1 = X-s ] ; then
	silent_inst=yes
	if [ X$2 = X ] ; then
		# no install directory specified
		exit 1
	fi
	if [ X$3 = X ] ; then
		# no user name specified
		exit 1
	fi
       if [ X$5 != X ] ; then
               patchID=$5
       else
               patchID=""
       fi

	install_dir=$2
	username=$3
	lcf_datdir=$4

	make_path $install_dir
	
	install_files
	write_start_script
	auto_start
	write_instance $instance
       if [ X$6 != X ] ; then
           rm -f $6
       fi

	exit 0

fi


clear
if [ X${os_name} != Xhpux ] && [ X${os_name} != Xaix ] && [ X${os_name} != Xsolaris ] && [ X${os_name} != Xlinux ] ; then
	echo "Platform $os_name is not supported."
	exit 1
fi

ok=0
while [ $ok = 0 ] ; do
	echo "To install the Tivoli Event Sink, you must accept the agreement written in the License file."
	echo $dashn "If you accept the agreement, enter Y.  [Y/N]: "
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ; then
		ok=1
	fi

	if [ X${ans} = Xn ] || [ X${ans} = XN ] ||
         [ X${ans} = Xno ] || [ X${ans} = XNO ] ; then
		exit 1
	fi
done

ok=0
default=/usr/eventsink
while [ $ok = 0 ] ; do
	echo $dashn "Installation directory [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		try_path=${ans}
	else
		try_path=${default}
	fi

	check_path ${try_path}
	if [ $? != 0 ] ; then
		echo $dashn "$try_path does not exist. Create it? [Y/N]: "
		check_response
		if [ $? = 0 ] ; then
			make_path $try_path
		fi
	fi

	check_path ${try_path}
	if [ $? = 0 ] ; then
		install_dir=${try_path}
		ok=1
	fi
done

ok=0
default=
interp=`print_interp`
while [ $ok = 0 ] ; do
	echo $dashn "LCF_DATDIR directory : "
	read ans
	if [ X${ans} != X ] ; then
		try_path=${ans}
		check_path ${try_path}
		if [ $? = 0 ] ; then
			mrt_dir=${try_path}/../../bin/${interp}/mrt
			check_path ${mrt_dir}
			if [ $? = 0 ] ; then
				lcf_datdir=${try_path}
				ok=1
			else
				echo "Unable to find the mrt directory at '${mrt_dir}'. Check the DAT directory you've inserted "
			fi
		else
			echo "DAT directory specified does not exist "
		fi
	fi
done


ok=0
while [ $ok = 0 ] ; do
	echo $dashn "Run the event sink as the following user: "
	read ans
	if [ X${ans} != X ] ; then
		username=$ans
		ok=1
	fi
done


ok=0
default=9444
while [ $ok = 0 ] ; do
	echo $dashn "Listening Port [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9])
			listening_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9])
			listening_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9])
			listening_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9][0-9])
			listening_port=$ans
			ok=1
			;;
		*)
			echo "Invalid port number -> " $ans
			;;
		esac
	else
		listening_port=$default
		ok=1
	fi
done

ok=0
default=50
while [ $ok = 0 ] ; do
	echo $dashn "Maximum Number of Events in Package [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9]*)
			max_num_evts=$ans
			ok=1
			;;
		*)
			echo "Invalid number -> " $ans
			;;
		esac
	else
		max_num_evts="${default}"
		ok=1
	fi
done

ok=0
default=40000
while [ $ok = 0 ] ; do
	echo $dashn "Maximum Buffer Size [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9]*)
			max_buf_sz=$ans
			ok=1
			;;
		*)
			echo "Invalid size -> " $ans
			;;
		esac
	else
		max_buf_sz="${default}"
		ok=1
	fi
done

# install files
echo $dashn "Installing event sink files..."
install_files
echo "OK"

# create configuration file
echo $dashn "Configuring the event sink..."
write_cfg_file
echo "OK"

# create and configure the start script
echo $dashn "Configuring the automatic restart... "
write_start_script
auto_start
echo "OK"

# register the application to the repository
echo $dashn "Registering the event sink... "
write_instance $instance
echo "OK"

ok=0
default="Y"
while [ $ok = 0 ] ; do
	echo $dashn "Start the event sink? [Y/N] [default=$default]: "
	check_response $default
	if [ $? = 0 ] ; then
		nohup ${install_dir}/eventsink.sh start 1>/dev/null 2>&1 &
	else
		echo "Event sink not started"
	fi
	ok=1
done
