#!/bin/sh


##        Licensed Materials - Property of IBM
##        (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved
##
##       US Government Users Restricted Rights - Use, duplication,
##       or disclosure restricted by GSA ADP Schedule Contract with
##       IBM Corp.
##

##
##   TFST 1.3 - script for upgrading the relay from version 1.2 to
##   the current version or for applying a patch to an installed version of
##   1.3 or 1.3.1.
##

version_major="1"
version_minor="3"
version_point="1"
version="$version_major.$version_minor.$version_point"
instance="relay"
repository_dir="/etc/Tivoli/TFST/$version/installed"
patchPrefix="1.3.1-TFS-"
patchNumber="0001"
patchID="$patchPrefix$patchNumber"
defaultPrevPatch="0001"

if echo -n | egrep -e -n >/dev/null ; then
    dashn=""
else
    dashn=-n
fi



print_os_name()
{
	if [ -f /bin/sun ] && /bin/sun ; then
		echo solaris
	elif [ `uname` = "AIX" ] ; then
		echo aix
	elif [ `uname` = "HP-UX" ] ; then
		echo hpux
	else
		if [ `uname` = "Linux" ] ; then
			echo linux
		else
			uname
		fi
	fi
}


check_response()
{
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ||
         [ "$1" = "Y" -a X${ans} = X ] ; then
		return 0
	fi

	return 1
}

check_path()
{
	if [ ! -d $1 ] ; then
		return 1
	fi

	if [ ! -f $1/relay ] ; then
		return 1
	fi

	if [ ! -f $1/relay.cfg ] ; then
		return 1
	fi

	return 0
}

# check_installed() will return 1 if a 1.3 or 1.3.1 instance
# of the relay is found.
check_installed()
{
    RepDir13="/etc/Tivoli/TFST/1.3/installed"
    RepDir131="/etc/Tivoli/TFST/$version/installed"
    relayFound=0
	
    # check if the installation repository directory exists
    if [ -d $RepDir131 ] ; then
        dummy=`ls $RepDir131/$instance* 1>/dev/null 2>/dev/null`
        if [ $? = 0 ] ; then
            instances131=`ls $RepDir131/$instance* | wc -l`
            if [ $instances131 -gt 0 ] ; then
                relayFound=1
            fi
        fi
    fi

    if [ $relayFound = 0 ] ; then
        if [ -d $RepDir13 ] ; then
            dummy=`ls $RepDir13/$instance* 1>/dev/null 2>/dev/null`
            if [ $? = 0 ] ; then
                instances13=`ls $RepDir13/$instance* | wc -l`
                if [ $instances13 -gt 0 ] ; then
                    relayFound=1
                fi
            fi
        fi
    fi

    return $relayFound
}

auto_start_remove()
{
OSNAME=`print_os_name`

case "$OSNAME" in
	aix)
		RCFILE=/etc/rc.$instance
		RCTAB=$instance

        set e+

		if [ -f $RCFILE ] ; then
			rm -rf $RCFILE
        fi

        rmitab $RCTAB

		set -e
	    ;;

	*)
	    for dir in /etc/rc.d/init.d /etc/init.d /usr/sbin/init.d /sbin/init.d; do
	        [ -d $dir ] && MRCD=$dir
	    done

	    rm -f $MRCD/$instance.rc
	    if [ -d /var/lock/subsys ]; then
		rm -f /var/lock/subsys/$instance
	    fi

	    find /etc/rc* -name S*$instance -exec rm {} \;
	    find /etc/rc* -name K*$instance -exec rm {} \;
	
    esac

    return 0
}

get_YN_response()
{
    ok=0
    response=0
    while [ $ok = 0 ] ; do
        echo $dashn $1
        read ans

        if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
           [ X${ans} = Xyes ] || [ X${ans} = XYES ] ; then
            response=0
            ok=1
        fi

        if [ X${ans} = Xn ] || [ X${ans} = XN ] ||
           [ X${ans} = Xno ] || [ X${ans} = XNO ] ; then
            response=1
            ok=1
        fi
    done

    return $response
}

get_relay_instance_to_patch()
{
    # Multiple instances of the relay can be installed on a single machine.
    # The instances can be identified by listing the relay-* files in
    # $repository_dir.  ***Starting with 1.3.1, there can be multiple
    # repository directories.***
    repository_dir131=$repository_dir
    installed131_file=$repository_dir131/$instance*
    if [ -d $repository_dir131 ] ; then
        dummy=`ls $installed131_file 1> /dev/null 2> /dev/null`
        if [ $? = 0 ] ; then
            echo "there must be relay* files in $repository_dir131."
            relay_131instance_count=`ls -la $installed131_file | wc -l`
        else
            echo "there are NO relay* files in $repository_dir131."
            relay_131instance_count=0
        fi
    else
        relay_131instance_count=0
    fi

    # $relay_instance_count now reflects the number of 1.3.1 relays installed.
    # count the number of 1.3 relays installed as well.
    repository_dir13="/etc/Tivoli/TFST/1.3/installed"
    installed13_file=$repository_dir13/$instance*

    if [ -d $repository_dir13 ] ; then
        dummy=`ls $installed13_file 1> /dev/null 2> /dev/null`
        if [ $? = 0 ] ; then
            echo "there must be relay* files in $repository_dir13."
            relay_13instance_count=`ls -la $installed13_file | wc -l`
        else
            echo "there are NO relay* files in $repository_dir13."
            relay_13instance_count=0
        fi
    else
       relay_13instance_count=0
    fi

    rInstDir="unknown"
    instToPatch="unknown"

    total_relay_count=`expr $relay_131instance_count + $relay_13instance_count`

    if [ $total_relay_count -gt 1 ] ; then
        validResponse=0
        while [ $validResponse = 0 ] ; do
            echo "The following instances of the Relay are installed:"
            if [ $relay_131instance_count -gt 0 ] ; then
                ls $installed131_file | while read LINE; do
                    rInstance=`echo $LINE | cut -d/ -f7`
                    rInstDir=`grep installation_dir $LINE | cut -d= -f2`
                    rInstPatchLevel=`grep version $LINE | awk '{ print $2 }'`
                    if [ X$rInstPatchLevel = X ] ; then
                        rInstPatchLevel="1.3.1-TFS-0001"
                    fi
                    echo "    $rInstance (patch level $rInstPatchLevel, installed in $rInstDir)"
                done
            fi

            if [ $relay_13instance_count -gt 0 ] ; then
                ls $installed13_file | while read LINE; do
                   rInstance=`echo $LINE | cut -d/ -f7`
                   rInstDir=`grep installation_dir $LINE | cut -d= -f2`
                   rInstPatchLevel=`grep version $LINE | awk '{ print $2 }'`
                   if [ X$rInstPatchLevel = X ] ; then
                       rInstPatchLevel="1.3-TFS-0001"
                   fi
                   echo "    $rInstance (patch level $rInstPatchLevel, installed in $rInstDir)"
                done
            fi

            echo "Please enter the Relay instance that you want to patch (for example, relay-2): "
            read ans

            # look for the instance to be patched in the 1.3 tree first
            if [ $relay_13instance_count -gt 0 ] ; then
                instToPatch=`ls $repository_dir13 | grep $ans`
                if [ X$instToPatch = X ] ; then
                    if [ $relay_131instance_count -gt 0 ] ; then
                        instToPatch=`ls $repository_dir131 | grep $ans`
                        if [ $instToPatch != X ] ; then
###                            pathToInstanceToPatch=`cat $repository_dir131/$ans | grep installation_dir | cut -d= -f2`
                            installedVersion="1.3.1"
                        fi
                    fi
                else
###                    pathToInstanceToPatch=`cat $repository_dir13/$ans | grep installation_dir | cut -d= -f2`
                    installedVersion="1.3"
                fi
            else
                instToPatch=`ls $repository_dir131 | grep $ans`
                if [ X$instToPatch != X ] ; then
###                   pathToInstanceToPatch=`cat $repository_dir131/$ans | grep installation_dir | cut -d= -f2`
                   installedVersion="1.3.1"
                fi
            fi

            if [ X$instToPatch = X ] ; then
                echo "'$ans' is not a valid response.  Please try again."
                echo
                echo
            else
                validResponse=1
            fi
        done
    else # only one relay instance, just return the instance value
        if [ $relay_13instance_count -gt 0 ] ; then
            instToPatch=`ls $installed13_file | cut -d/ -f7`
            installedVersion="1.3"
        else
            instToPatch=`ls $installed131_file | cut -d/ -f7`
            installedVersion="1.3.1"
        fi
    fi

    # $instance is a global variable
    instance=$instToPatch
}

get_install_dir()
{
    instDir=`grep installation_dir $installFile`

    if [ X"$instDir" = X ] ; then
        echo "The Relay installation directory cannot be determined."
        echo "Cannot continue, exiting..."
        exit 1
    else
        instDir=`grep installation_dir $installFile | cut -d= -f2`
    fi

    # IY47957: trailing slashes cause problems so remove them
    #    (but not for bare slash ("/"))
    instDir=`echo $instDir | sed 's,\(.\)//*$,\1,'`

    check_path ${instDir}
    if [ $? != 0 ] ; then
        echo "The Relay was not found in the directory $instDir."
        echo "Nothing to patch, exiting..."
        exit 1
    else
        install_dir=${instDir}
    fi
}

get_username()
{
    # TFST 1.3 GA does not have 'runas' line in the install history file;
    # the userID that relay will run as is the owner of the relay binary.
    # Patched versions of TFST 1.3 should have a 'runas' line in
    # the install history file to identify the userID the relay runs as,
    # since install versions after TFST 1.3 GA will set the owner of
    # the relay binary to root.
    runasName=`grep runas $installFile`
    if [ X${runasName} = X ] ; then
        username=`ls -l $install_dir/relay | awk '{ print $3 }'`
    else
        username=`grep runas $installFile | cut -d= -f2`
    fi

    if [ X${username} = X ] ; then
        echo "Unable to identify the user of the Relay directory ($install_dir/relay)."
        echo "Cannot continue, exiting..."
        exit 1
    fi
}

check_patch_level()
{
    # get_relay_instance_to_patch sets $instance to the relay instance
    # the user chooses to be patched. $pathToInstanceToPatch is also
    # set by get_relay_instance_to_patch.
    get_relay_instance_to_patch;

    installFile="/etc/Tivoli/TFST/$installedVersion/installed/$instance"
    _patchPrefix="$installedVersion-TFS-"

    # The $installFile for TFST 1.3 GA does not contain the $patchPrefix
    # string.  Installs or upgrades of subsequent patches will write
    # the patchID to the $installFile.
    patchApplied=`grep $_patchPrefix $installFile`

     # when $_patchPrefix is not found in $installFile, $? will be non-zero
     if [ $? != 0 ] ; then
        # grep failed to find $_patchPrefix in $installFile
        if [ $installedVersion != $version ] ; then
            # assume this is 1.3 GA
            previousPatch=$_patchPrefix$defaultPrevPatch
        else
            # this is a 1.3.1 install, but the patch level is unknown.
            # Ask if install should continue.
            echo "The patch level of $instance cannot be determined."
            get_YN_response "Do you want to continue installing this patch? [Y/N] ";
            if [ $? = 0 ] ; then
                previousPatch=$_patchPrefix"Unknown"
            else
                echo "The patch will not be applied."
                exit 1
            fi
        fi
    else
        # We're in this leg if _patchPrefix (e.g. 1.3.1-TFS) was found
        # $installFile.  Get the patch string from $installFile and
        # then parse it further.  The following code breaks the
        # patch string into its component parts.

        # $installedPatch will be something like 1.3-TFS-0002
        installedPatch=`grep version $installFile | awk '{ print $2 }'`

        # $installedPatchNum will be something like 0002
        installedPatchNum=`echo $installedPatch | cut -d- -f3`

        # $instPfx1 will be something like 1.3.
        instPfx1=`echo $installedPatch | cut -d- -f1`
        installedPatchPrefix="$instPfx1-TFS-"

        # $installedPatchNum should never be blank, but handle it just in case.
        if [ X$installedPatchNum = X ] ; then
            echo "The installed patch level is $_patchPrefix0001."
            previousPatch="$_patchPrefix0001"

        else
            previousPatch=$_patchPrefix$installedPatchNum
            echo "The installed patch level is $previousPatch."

            # here, installedPatchNum could be 0002 (for 1.3-TFS-0002).
            # this will be less than $patchNumber (0001).
            # Don't do the next two comparisons if $installedVersion != $version
            if [ $installedVersion = $version ] ; then

                if [ $installedPatchNum = $patchNumber ] ; then
                    echo "The patch level of the installed Relay ($_patchPrefix$installedPatchNum) is the same as this patch."
                    get_YN_response "Do you want to continue installing this patch? [Y/N] ";
                    if [ $? = 0 ] ; then
                        previousPatch=$_patchPrefix$installedPatchNum
                    else
                        echo "The patch will not be applied."
                        exit 1
                    fi
                elif [ $installedPatchNum -gt $patchNumber ] ; then
                    echo "The patch level of the installed Relay ($_patchPrefix$installedPatchNum) is higher than this patch."
                    echo "Installing this patch will result in patch $patchID replacing what is currently installed ($_patchPrefix$installedPatchNum)."
                    get_YN_response "Do you want to continue installing this patch? [Y/N] ";
                    if [ $? = 0 ] ; then
                        previousPatch=$_patchPrefix$installedPatchNum
                    else
                        echo "The patch will not be applied."
                        exit 1
                    fi
                fi
            fi
        fi
    fi

    # set install_dir (relay instance install location)
    get_install_dir $installFile

    # set username (userID that relay instance will run as)
    get_username $installFile

    installedRepositoryFile=$installFile
}

################################################
#
#    install script starts here
#
################################################

clear
work_dir=`pwd`

#
# check platform
#
os_name=`print_os_name`
if [ X${os_name} != Xhpux ] && [ X${os_name} != Xaix ] && [ X${os_name} != Xsolaris ] && [ X${os_name} != Xlinux ] ; then
	echo "Platform $os_name is not supported."
	exit 1
fi

#
# ask for license agreement
#
ok=0

echo "To install the Tivoli Relay, you must accept the agreement written in the License file."

get_YN_response "If you accept the agreement, enter Y.  [Y/N]: ";
if [ $? = 1 ] ; then
    exit 1
fi

#
# See if 1.3 is installed.  If so, patch.  Otherwise upgrade from 1.2
# to this patch level of 1.3.
#
unknown="unknown"
install_dir=$unknown
username=$unknown
previousPatch=$unknown
install13Patch=0
check_installed;
if [ $? = 1 ] ; then
     echo "Tivoli Relay, version 1.3 or later, is already installed."
     echo "Checking Relay patch level..."
       install13Patch=1
fi

if [ $install13Patch = 1 ] ; then
    # On return from check_patch_level, $install_dir, $username, $instance have
    # all been set.  check_patch_level exits if the currently installed patch
    # level is >= this patch level and the user declines to proceed.
    installedVersion=$unknown
    check_patch_level;
fi

#
# If upgrade from 1.2, ask for installation directory
#
ok=0
if [ $install13Patch = 0 ] ; then
    while [ $ok = 0 ] ; do
	echo $dashn "Relay 1.2 installation directory: "
	read ans
	if [ X${ans} != X ] ; then
		# check if the path exists
		try_path=${ans}
		check_path ${try_path}
		if [ $? != 0 ] ; then
			echo $dashn "The relay was not found in the directory: $try_path"
		else
			install_dir=${try_path}

			# IY47957: trailing slashes cause problems so remove them
                        #    (but not for bare slash ("/"))
                        install_dir=`echo $install_dir | sed 's,\(.\)//*$,\1,'`

			ok=1
		fi
	fi
    done

    #
    # get the user name
    #
    username=`ls -l $install_dir/relay | awk '{ print $3 }'`
    if [ X${username} = X ] ; then
	echo $dashn "Unable to identify the user of the directory you specified."
	exit 1
    fi
fi

#
# Confirm upgrade/patch
#
ok=0
while [ $ok = 0 ] ; do
   if [ $install13Patch = 1 ] ; then
       backupDir=${install_dir}.${previousPatch}
       echo
       echo "The configuration and all the data of the current Relay installation will be backed up to '$backupDir'."
       echo "If the directory '$backupDir' exists, the directory and all of its current contents will be removed."
       echo "Configuration data from the current Relay installation will be used by the patch."
       get_YN_response "Do you want to apply the patch now? [Y/N] ";
   else
       backupDir=${install_dir}.1.2
	echo
	echo "A previous version of the relay has been detected on the machine."
	echo "The configuration and all the data of the previous version will be backed up to '${install_dir}.1.2'."
	echo "It will be used by the new version."
       get_YN_response "Do you want to upgrade to the new version? [Y/N] ";
   fi

   if [ $? = 1 ] ; then
       exit 1
   fi
done

#
# back up the appropriate directory tree
#
if [ -d "${backupDir}" ] ; then
	rm -rf "${backupDir}"
fi
cp -rf ${install_dir} "${backupDir}"

#
# remove old directory (except cfg)
#
$install_dir/relay.sh stop > /dev/null
rm -f $install_dir/relay
rm -f $install_dir/*.log
rm -f $install_dir/*.pid
rm -f $install_dir/relay.sh
rm -f $install_dir/C/*
rmdir $install_dir/C
if [ $install13Patch = 1 ] ; then
    rm -f $install_dir/*.log.bak > /dev/nul
    rm -f $install_dir/uninstall.sh > /dev/nul
    rm -rf $install_dir/de > /dev/nul
    rm -rf $install_dir/es > /dev/nul
    rm -rf $install_dir/fr_FR > /dev/nul
    rm -rf $install_dir/it > /dev/nul
    rm -rf $install_dir/ja_JP > /dev/nul
    rm -rf $install_dir/ko > /dev/nul
    rm -rf $install_dir/pt_BR > /dev/nul
    rm -rf $install_dir/zh_CN > /dev/nul
    rm -rf $install_dir/zh_TW > /dev/nul
    # remove install history instance file (only
    # exists in 1.3).  Install fails if this
    # file exists.
    rm -f $repository_dir/$instance > /dev/nul
fi
auto_start_remove

if [ $install13Patch = 0 ] ; then
    # This is an upgrade from 1.2 to 1.3.  In 1.3, there can be multiple
    # relay instances installed, so 1.3 relay instances are numbered.
    # Set $instance to indicate this is the first relay instance installed.
    # NOTE that until now, $instance=="relay", which is the correct value
    # to use to remove 1.2 autostart files, etc.
    instance="relay-1"
fi

#
# launch the silent install
#
# if upgrading from 1.3 to 1.3.1, need to tell install.sh to remove
# the existing install file /etc/Tivoli/TFST/1.3/installed/relay-n
if [ $install13Patch = 1 -a $version != $installedVersion ] ; then
    # pass in name of existing repository file so install can remove it
    ./install.sh -s ${install_dir} ${username} ${patchID} ${instance} ${installedRepositoryFile}
else
    ./install.sh -s ${install_dir} ${username} ${patchID} ${instance}
fi

if [ $install13Patch = 1 ] ; then
    echo "Relay patch $patchID applied successfully."
else
    echo "Relay successfully upgraded."
fi

#
# ask for relay start-up
#
ok=0
default="Y"
while [ $ok = 0 ] ; do
	echo $dashn "Start the relay? [Y/N] [default=$default]: "
	check_response $default
	if [ $? = 0 ] ; then
		nohup ${install_dir}/relay.sh start 1>/dev/null 2>&1 &
	else
		echo "Relay not started"
	fi
	ok=1
done
