#!/bin/sh

##
##        Licensed Materials - Property of IBM
##        (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved
##
##       US Government Users Restricted Rights - Use, duplication,
##       or disclosure restricted by GSA ADP Schedule Contract with
##       IBM Corp.
##

##
##   TFST 1.3.1 - relay install script
##

version="1.3.1"
instance="relay-1"
patchID="1.3.1-TFS-0001"
repository_dir="/etc/Tivoli/TFST/$version/installed"

if echo -n | egrep -e -n >/dev/null ; then
	dashn=""
else
	dashn=-n
fi

# create the installation repository directory
if [ ! -d $repository_dir ] ; then
	mkdir -p $repository_dir
fi

if [ ! -d $repository_dir ] ; then
	echo "Unable to create the repository folder '$repository_dir'"
	return 0
fi

print_os_name()
{
	if [ -f /bin/sun ] && /bin/sun ; then
		echo solaris
	elif [ `uname` = "AIX" ] ; then
		echo aix
	elif [ `uname` = "HP-UX" ] ; then
		echo hpux
	else
		if [ `uname` = "Linux" ] ; then
			echo linux
		else
			uname
		fi
	fi
}


check_response()
{
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ||
         [ "$1" = "Y" -a X${ans} = X ] ; then
		return 0
	fi

	return 1
}

#
# This function replaces the !!xxx!! patterns
# in the uninstall script.
#
# $1 is the full path uninstall file
#
patch_uninstall_script()
{
	if [ ! -f $1 ] ; then
		echo "Uninstall file '$1' not found, the uninstall script will be not available"
		return 0
	fi

  	sed_cmd=" sed -e s#!!INSTALL_DIR!!#$install_dir#g
     -e s/!!INSTANCE!!/$instance/g "
	$sed_cmd $1 > $1.new
	rm -f $1
	mv $1.new $1

	return 1		
}


write_instance()
{
	installed_file=$repository_dir/$1

	if [ -f $installed_file ] ; then
		echo "Instance $1 already installed"
		return 0
	fi

	# create the installation info file
cat >$installed_file <<EOF
installation_dir=$install_dir
installation_time=`date`
version=$version $patchID
runas=$username
EOF

	return 1		
}


get_free_instance()
{
	idx=1
	found=0
	while [ $found -ne 1 ] ; do
		if [ -f $repository_dir/relay-${idx} ] ; then
			idx=`expr ${idx} + 1`
		else
			found=1
		fi
	done
	instance="relay-${idx}"	
	return 1
}



check_path()
{
	if [ -d $1 ] ; then
		return 0
	fi

	return 1
}

make_path()
{
	check_path $1
	if [ $? != 0 ] ; then
		mkdir -p $1
	fi
	check_path $1
	return $?
}



auto_start()
{

instance=$1
start_script=relay.sh
OSNAME=`print_os_name`

ETRC=$install_dir/$start_script
CRCF="/etc/rc3.d/S999$instance \
      /etc/rc0.d/K49$instance \
      /etc/rc1.d/K49$instance \
      /etc/rc2.d/K49$instance"
DEVNULL=/dev/null
DATE=`date`
# add /usr/sbin to path for AIX to find mkitab
PATH=$PATH:/usr/sbin


case "$OSNAME" in

linux)
    # SuSE looks for kill scripts in prev level. All other look in new runlevel
    if [ -f /etc/SuSE-release ]; then
      CRCF="/etc/rc.d/rc2.d/S999$instance \
            /etc/rc.d/rc3.d/S999$instance \
            /etc/rc.d/rc4.d/S999$instance \
            /etc/rc.d/rc5.d/S999$instance \
            /etc/rc.d/rc2.d/K9$instance \
            /etc/rc.d/rc3.d/K9$instance \
            /etc/rc.d/rc4.d/K9$instance \
            /etc/rc.d/rc5.d/K9$instance"
    else
      CRCF="/etc/rc.d/rc2.d/S999$instance \
            /etc/rc.d/rc3.d/S999$instance \
            /etc/rc.d/rc4.d/S999$instance \
            /etc/rc.d/rc5.d/S999$instance \
            /etc/rc.d/rc0.d/K9$instance \
            /etc/rc.d/rc1.d/K9$instance \
            /etc/rc.d/rc6.d/K9$instance"
    fi
    ;;
hpux)
      CRCF="/sbin/rc3.d/S999$instance \
            /sbin/rc0.d/K9$instance \
            /sbin/rc1.d/K9$instance \
            /sbin/rc2.d/K9$instance"
    ;;
esac

#echo "RC files are: $CRCF"

#################
# Setup autostart
#################

case "$OSNAME" in
	aix)
		RCFILE=/etc/rc.$instance
		RCTAB=$instance
		INITTAB=/etc/inittab
		INITSAVE=$INITTAB.before.$instance

		if [ ! -f $RCFILE ]
		then
			set +e
			
            # create the rc.$instance file
            cat >> $RCFILE <<EOF
#!/bin/sh
#
#	Start the Tivoli Relay ($instance)
#
if [ -f $ETRC ]; then
	$ETRC start
fi
EOF

			if [ $? -ne 0 ]
			then
				rm -rf $RCFILE
				echo "Unable to create the file '$RCFILE'"
				return 1
			fi

            # set permissions for rc.$instance file
			chmod 0774 $RCFILE

			if [ $? -ne 0 ]
			then
				rm -rf $RCFILE
				echo "Unable to set permissions (0774) for the file '$RCFILE'"
				return 1
			fi

			if lsitab rcnfs >/dev/null 2>&1
			then
				RCENTRY=rcnfs
			else
				RCENTRY=rctcpip
			fi

			#
			#  Okay - we've successfully added the rc startup file,
			# need to modify /etc/inittab
			#
			cp $INITTAB $INITSAVE
			mkitab -i $RCENTRY "$RCTAB:2:wait:$RCFILE > /dev/console 2>&1 # Tivoli Relay ($instance)"
			if [ $? -ne 0 ]
				then
				rmitab $RCTAB
				rm -rf $RCFILE
				cp $INITSAVE $INITTAB
				rm -rf $INITSAVE
				echo "Unable to modify /etc/inittab"
				return 1
			fi
			set -e
		fi
	    ;;
	*)
	    for dir in /etc/rc.d/init.d /etc/init.d /usr/sbin/init.d /sbin/init.d; do
	        [ -d $dir ] && MRCD=$dir
	    done

	    if [ "$OSNAME" = "linux" ]; then
	        MRCF="$MRCD/$instance"
		if [ -d /var/lock/subsys ]; then
			touch /var/lock/subsys/$instance
		fi
	    else
	        MRCF="$MRCD/$instance.rc"
	    fi

	    if [ ! -d $MRCD ]; then
	        MRCF="$ETRC"
	    fi
	    if test -f $MRCF && cmp -s $MRCF $ETRC >$DEVNULL 2>&1; then
	        echo "$MRCF Tivoli $proxy daemon autostart file is already present."
	    else
	        rm -f $MRCF
	    	cp -p $ETRC $MRCF
	    fi
	    for i in $CRCF; do
		if [ -f $i ]; then
		    rm -f $i;
		fi
		# the convention for hpux and sgi is to always
		# symlink startup scripts from init.d/script to
		# the corresponding rc?.d/script
		case "$OSNAME" in
		    hpux|linux)
			di=`dirname $i`
			if [ X"$di" != X"" -a -d "$di" ] ; then
				ln -s $MRCF $i
			fi
			;;
		    *)
	        	ln $MRCF $i >$DEVNULL 2>&1
			if [ $? -ne 0 ]; then
			    ln -s $MRCF $i
			fi
			;;
		esac
	    done
            ;;
    esac

    return 0
}

install_files()
{
        # set permissions on installed files
        chown $username $install_dir
        chmod 750 $install_dir
        cp relay uninstall.sh $install_dir/
        chmod 555 $install_dir/relay

        for langDir in C it de es fr_FR pt_BR zh_CN zh_TW ja_JP ko ; do
            cp -rf $langDir $install_dir/
            chmod 755 $install_dir/$langDir
            chmod 644 $install_dir/$langDir/*
        done

        patch_uninstall_script $install_dir/uninstall.sh
        chmod 500 $install_dir/uninstall.sh
}

write_cfg_file()
{
logfile_path="relay.log"

# create the configuration file for the proxy
cat >$install_dir/relay.cfg <<EOF
[relay]

# Timeout interval (in seconds) for TCP/IP operations to succeed/fail
#tcpip-timeout

[log]

# Sets the level of debug [0-11] (default = 3)
debug-level = 3

# Full path to the log file where the Relay messages are written (default = <STDERR>)
log-file = $logfile_path

# The max size of the log file in MegaBytes (if 0 the log file size is unlimited) (default = 1)
max-size = 1


[communication-layer]
# The network interface to use for listening traffic (from Endpoint Proxy or parent Relay)
# parent-local-host =

# The port the Relay listens on for Endpoint Proxy or parent Relay traffic
parent-local-port = $parent_local_port

# The parent host name
parent-remote-host = $parent_remote_host

# The port the parent host listens on for Relay traffic
parent-remote-port = $parent_remote_port

# The communication interface used to connect to parent
parent-cm-type = $cm_type_parent


# The network interface to use for listening traffic (from child Relay(s) or Gateway Proxy(s))
#children-local-host =

# The port the Relay listens on for traffic from children
children-local-port = $children_local_port

# List of children hosts (e.g. hostname1+port1[;hostname2+port2 ...])
children-remote-list = $peers

# The communication interface used to connect to children
children-cm-type = $cm_type_children

[parent-cm-info]
# Connection mode (unidirectional only, may be "client" or "server")
connection-mode = $cm_mode_parent

[children-cm-info]
# Connection mode (unidirectional only, may be "client" or "server")
connection-mode = $cm_mode_children
EOF

chmod 644 $install_dir/relay.cfg
}

write_start_script()
{
nfile=unlimited
ulimit_solaris2="ulimit -n $nfile"
ulimit_linux="ulimit -n 32768"
# for HP ulimit -n is not available
ulimit_hpux=""

if [ X${os_name} = Xsolaris ] ; then
	ulimit_cmd=${ulimit_solaris2}
fi

if [ X${os_name} = Xlinux ] ; then
	ulimit_cmd=${ulimit_linux}
fi

if [ X${os_name} = Xhpux ] ; then
	ulimit_cmd=${ulimit_hpux}
	nohup_cmd=nohup
fi

# create the script to run the proxy
cat >$install_dir/relay.sh <<EOF
#!/bin/sh
NLSPATH=$install_dir/%L/%N.cat:$install_dir/%l/%N.cat:$install_dir/C/%N.cat\${NLSPATH:+":\$NLSPATH"}
export NLSPATH
PATH=/bin:/usr/bin:/usr/ucb:/usr/sbin:${install_dir}:$PATH
export PATH

${ulimit_cmd}

[ -d "$install_dir" ] && cd $install_dir

case "\$1" in
	
	stop_msg)
		shift
		echo "Stopping the relay daemon"
		;;
	stop)
		shift
		if [ -f $install_dir/relay.pid ] ; then
			pid=\`sed -n 1p $install_dir/relay.pid\`
			if [ -n "\$pid" ] ; then
				
				pidlist=\`ps -el | grep relay | grep -v grep | awk '{print \$4}'\`
				
				for temppid in \$pidlist
				do
					if [ \$temppid -eq \$pid ] ; then
						rm -f $install_dir/relay.pid
						kill -15 \$pid
						exit \$?
					fi
				done
				echo "The process id in $install_dir/relay.pid is not an relay process id."
				echo "Unable to get process id of the relay daemon (pid)."
			else
				echo "Unable to get process id of the relay daemon (pid)."
				exit 1
			fi
		else
			echo "Unable to get process id of the relay daemon (relay.pid)."
			exit 1
		fi
		;;
	start_msg)
		shift
		echo "Starting the relay daemon"
		;;
	start|*)
		if [ "\$1" = "start" ] ; then
			shift
		fi

		if [ -f $install_dir/relay.pid ] ; then
			pid=\`sed -n 1p $install_dir/relay.pid\`
			if [ -n "\$pid" ] ; then
				
				pidlist=\`ps -el | grep relay | grep -v grep | awk '{print \$4}'\`
				
				for temppid in \$pidlist
				do
					if [ \$temppid -eq \$pid ] ; then
						echo "relay for this environment already running under pid \$pid"
						exit 1
					fi
				done
			fi
		fi

		currentUser=\`whoami\`
		if [ "\$currentUser" = "$username" ] ; then
			$nohup_cmd $install_dir/relay $install_dir &
		else
			$nohup_cmd su - $username -c "NLSPATH=\$NLSPATH $install_dir/relay $install_dir" &
		fi

		;;
esac

EOF


# set permissions on the script
chmod 500 $install_dir/relay.sh
}

################################################
#
#    install script starts here
#
################################################

work_dir=`pwd`
os_name=`print_os_name`

#
# Check if the installation must be silent (-s).
# In this case is expected as
#
#	2nd parameter: the installation directory
#	3nd parameter: the user name
#      4th parameter: the patch ID
#      5th parameter: the relay instance
#      6th parameter: existing repository file to be deleted
#                     if install succeeds
#
# and no configuration file will be written.
#
if [ X$1 = X-s ] ; then
	silent_inst=yes
	if [ X$2 = X ] ; then
		# no install directory specified
		exit 1
	fi
	if [ X$3 = X ] ; then
		# no user name specified
		exit 1
	fi
       if [ X$4 != X ] ; then
               patchID=$4
       else
               patchID=""
       fi

	install_dir=$2
	username=$3
       instance=$5

	make_path $install_dir
	
	install_files
	write_start_script
	auto_start $instance
	write_instance $instance
       if [ X$6 != X ] ; then
           rm -f $6
       fi

	exit 0

fi

clear
if [ X${os_name} != Xhpux ] && [ X${os_name} != Xaix ] && [ X${os_name} != Xsolaris ] && [ X${os_name} != Xlinux ] ; then
	echo "Platform $os_name is not supported."
	exit 1
fi


get_free_instance

ok=0
while [ $ok = 0 ] ; do
	echo "To install the Tivoli Relay, you must accept the agreement written in the License file."
	echo $dashn "If you accept the agreement, enter Y.  [Y/N]: "
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ; then
		ok=1
	fi

	if [ X${ans} = Xn ] || [ X${ans} = XN ] ||
         [ X${ans} = Xno ] || [ X${ans} = XNO ] ; then
		exit 1
	fi
done


ok=0
default=/usr/$instance
while [ $ok = 0 ] ; do
	echo $dashn "Installation directory [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		try_path=${ans}
	else
		try_path=${default}
	fi

	check_path ${try_path}
	if [ $? != 0 ] ; then
		echo $dashn "$try_path does not exist. Create it? [Y/N]: "
		check_response
		if [ $? = 0 ] ; then
			make_path $try_path
		fi
	fi

	check_path ${try_path}
	if [ $? = 0 ] ; then
		install_dir=${try_path}
		ok=1
	fi
done

ok=0
while [ $ok = 0 ] ; do
	echo $dashn "Run the relay as the following user: "
	read ans
	if [ X${ans} != X ] ; then
		username=$ans
		ok=1
	fi
done


echo "-------------------------------"
echo "Relay-Parent Connection Options"
echo "-------------------------------"

ok=0
while [ $ok = 0 ] ; do
	echo $dashn "Relay port: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9])
			parent_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9])
			parent_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9])
			parent_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9][0-9])
			parent_local_port=$ans
			ok=1
			;;
		*)
			echo "Invalid port number -> " $ans
			;;
		esac
	else
		ok=0
	fi
done

ok=0
while [ $ok = 0 ] ; do
	echo $dashn "Relay or endpoint proxy hostname: "
	read ans
	if [ X${ans} != X ] ; then
		parent_remote_host=$ans
		ok=1
	else
		ok=0
	fi
done


ok=0
while [ $ok = 0 ] ; do
	echo $dashn "Relay or Endpoint Proxy Port: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9])
			parent_remote_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9])
			parent_remote_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9])
			parent_remote_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9][0-9])
			parent_remote_port=$ans
			ok=1
			;;
		*)
			echo "Invalid port number -> " $ans
			;;
		esac
	else
		ok=0
	fi
done

ok=0
cm_mode_parent=server
while [ $ok = 0 ] ; do
	echo $dashn "Specify how relay connects to parent relay or endpoint proxy [0=bidirectional, 1=unidirectional initiator, 2=unidirectional listener]: "
	read ans
    if [ X${ans} = X0 ] ; then
        cm_type_parent=cm-tcp-bidirectional
        ok=1
    elif [ X${ans} = X1 ] ; then
        cm_type_parent=cm-tcp-unidirectional
        cm_mode_parent=client
        ok=1
    elif [ X${ans} = X2 ] ; then
        cm_type_parent=cm-tcp-unidirectional
        cm_mode_parent=server
        ok=1
    else
    	echo "Invalid choice -> " $ans
	fi
done



echo "---------------------------------"
echo "Relay-Children Connection Options"
echo "---------------------------------"

ok=0

while [ $ok = 0 ] ; do
	echo $dashn "Relay port: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9])
			children_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9])
			children_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9])
			children_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9][0-9])
			children_local_port=$ans
			ok=1
			;;
		*)
			echo "Invalid port number -> " $ans
			;;
		esac
	else
		ok=0
	fi
done


done=0
while [ $done = 0 ] ; do
	ok=0
	while [ $ok = 0 ] ; do
		echo $dashn "Relay or gateway proxy hostname: "
		read ans
		if [ X${ans} != X ] ; then
			peer_host=$ans
			ok=1
		else
			ok=0
		fi
	done

	ok=0
	while [ $ok = 0 ] ; do
		echo $dashn "Relay or gateway proxy port: "
		read ans
		if [ X${ans} != X ] ; then
			case $ans in
			[1-9][0-9])
				peer_port=$ans
				ok=1
				;;
			[1-9][0-9][0-9])
				peer_port=$ans
				ok=1
				;;
			[1-9][0-9][0-9][0-9])
				peer_port=$ans
				ok=1
				;;
			[1-9][0-9][0-9][0-9][0-9])
				peer_port=$ans
				ok=1
				;;
			*)
				echo "Invalid port number -> " $ans
				;;
			esac
		else
			ok=0
		fi
	done

	if [ X${peers} != X ] ; then
		peers="${peers};${peer_host}+${peer_port}"
	else
		peers="${peer_host}+${peer_port}"
	fi

	default=N
	echo $dashn "Enter more destinations? [Y/N] [default=$default]: "
	check_response
	if [ $? != 0 ] ; then
		done=1
	fi
done

#
# Connection manager type
#
ok=0
cm_mode_children=client
while [ $ok = 0 ] ; do
	echo $dashn "Specify how relay connects to child destinations [0=bidirectional, 1=unidirectional initiator, 2=unidirectional listener]: "
	read ans
    if [ X${ans} = X0 ] ; then
        cm_type_children=cm-tcp-bidirectional
        ok=1
    elif [ X${ans} = X1 ] ; then
        cm_type_children=cm-tcp-unidirectional
        cm_mode_children=client
        ok=1
    elif [ X${ans} = X2 ] ; then
        cm_type_children=cm-tcp-unidirectional
        cm_mode_children=server
        ok=1
    else
    	echo "Invalid choice -> " $ans
	fi
done

# install files
echo $dashn "Installing relay files..."
install_files
echo "OK"

# create configuration file
echo $dashn "Configuring the relay..."
write_cfg_file
echo "OK"

# create and configure the start script
echo $dashn "Configuring the automatic restart... "
write_start_script
auto_start $instance
echo "OK"

# register the application to the repository
echo $dashn "Registering the relay... "
write_instance $instance
echo "OK"


ok=0
default="Y"
while [ $ok = 0 ] ; do
	echo $dashn "Start the relay? [Y/N] [default=$default]: "
	check_response $default
	if [ $? = 0 ] ; then
		nohup ${install_dir}/relay.sh start 1>/dev/null 2>&1 &
	else
		echo "Relay not started"
	fi
	ok=1
done
